@@ -38,16 +38,19 @@ from .metrics import (
 )
 from .strategy import Strategy, BuyAndHold
 from .exceptions import BacktestRunError, LongShortLiquidationError, NegativeValueError
-from .utils import is_notebook
+from .utils import is_notebook, _cls
 
+# matplotlib is not a strict requirement, only needed for live_plot
 try:
     import pylab as pl
     import matplotlib.pyplot as plt
+    import matplotlib
 
     plt_exists = True
 except ImportError:
     plt_exists = False
 
+# tqdm is not a strict requirement
 try:
     from tqdm import tqdm
 
@@ -119,11 +122,6 @@ class StrategySequence:
         return len(self.bt._get_bts())
 
 
-def _cls():
-    clear_output(wait=True)
-    os.system("cls" if os.name == "nt" else "clear")
-
-
 class Position:
     """Tracks a single position in a portfolio or trade history.
     """
@@ -134,87 +132,76 @@ class Position:
         symbol: str,
         date: datetime.date,
         event: str,
-        num_shares: int,
+        nshares: int,
         uid: str,
+        fee: float,
         slippage: float = None,
     ):
         self.symbol = symbol
         self.date = date
         self.event = event
-        self.num_shares_int = num_shares
+        self._nshares_int = nshares
         self.start_price = bt.price(symbol)
         if slippage is not None:
-            if num_shares < 0:
+            if nshares < 0:
                 self.start_price *= 1 + slippage
-            if num_shares > 0:
+            if nshares > 0:
                 self.start_price *= 1 - slippage
         self._slippage = slippage
-        self.bt = bt
-        self.frozen = False
-        self.df_cols = [
-            "symbol",
-            "date",
-            "event",
-            "order_type",
-            "profit_loss_abs",
-            "profit_loss_pct",
-            "price",
-        ]
-        self.end_date = None
-        self.end_event = None
-        self.uid = uid
+        self._bt = bt
+        self._frozen = False
+        self._uid = uid
+        self.fee = fee
+
+    def _attr(self):
+        return [attr for attr in dir(self) if not attr.startswith("_")]
 
     def __repr__(self) -> str:
-        t = self.order_type
-        result = {
-            "symbol": self.symbol,
-            "date & event": str(self.date) + " " + self.event,
-            "type": t,
-            "shares": self.num_shares,
-            "profit/loss (abs)": f"{self.profit_loss_abs:.2f}",
-            "profit/loss (%)": f"{self.profit_loss_pct:.2f}",
-            "price": f"{self.price:.2f}",
-        }
-        if self.frozen:
-            result["end date & event"] = str(self.end_date) + " " + self.end_event
+        result = {}
+        for attr in self._attr():
+            val = getattr(self, attr)
+            if isinstance(val, float):
+                result[attr] = f"{val:.2f}"
+            else:
+                result[attr] = str(val)
         return json.dumps(result, sort_keys=True, indent=2)
 
     @property
-    def short(self) -> bool:
+    def _short(self) -> bool:
         """True if this is a short position.
         """
-        return self.num_shares_int < 0
+        return self._nshares_int < 0
 
     @property
-    def long(self) -> bool:
+    def _long(self) -> bool:
         """True if this is a long position.
         """
-        return self.num_shares_int > 0
+        return self._nshares_int > 0
 
     @property
     def value(self) -> float:
         """Returns the current market value of the position.
         """
-        if self.short:
+        if self._short:
             old_val = self.initial_value
-            cur_val = self.num_shares * self.price
+            cur_val = self.nshares * self.price
             return old_val + (old_val - cur_val)
-        if self.long:
-            return self.num_shares * self.price
+        if self._long:
+            return self.nshares * self.price
 
     @property
     def price(self) -> float:
         """Returns the current price if the position is held in a portfolio.
         Returns the last price if the position was liquidated and is part of a trade history.
         """
-        if self.frozen:
-            result = self.bt.prices[self.symbol, self.end_date][self.end_event]
+        if self._frozen:
+            result = self._bt.prices[self.symbol, self.end_date][self.end_event]
         else:
-            result = self.bt.price(self.symbol)
+            result = self._bt.price(self.symbol)
         if self._slippage is not None:
-            if self.short:
+            if self._short:
                 result *= 1 - self._slippage
-            if self.long:
+            if self._long:
                 result *= 1 + self._slippage
         return result
 
@@ -222,19 +209,19 @@ class Position:
     def value_pershare(self) -> float:
         """Returns the value of the position per share.
         """
-        if self.long:
+        if self._long:
             return self.price
-        if self.short:
+        if self._short:
             return self.start_price + (self.start_price - self.price)
 
     @property
     def initial_value(self) -> float:
-        """Returns the initial value of the position.
+        """Returns the initial value of the position, including fees.
         """
-        if self.short:
-            return self.num_shares * self.start_price
-        if self.long:
-            return self.num_shares * self.start_price
+        if self._short:
+            return self.nshares * self.start_price + self.fee
+        if self._long:
+            return self.nshares * self.start_price + self.fee
 
     @property
     def profit_loss_pct(self) -> float:
@@ -251,33 +238,32 @@ class Position:
         return self.value - self.initial_value
 
     @property
-    def num_shares(self) -> int:
+    def nshares(self) -> int:
         """Returns the number of shares in the position.
         """
-        return abs(self.num_shares_int)
+        return abs(self._nshares_int)
 
     @property
     def order_type(self) -> str:
         """Returns "long" or "short" based on the position type.
         """
         t = None
-        if self.short:
+        if self._short:
             t = "short"
-        if self.long:
+        if self._long:
             t = "long"
         return t
 
     def _remove_shares(self, n):
-        if self.short:
-            self.num_shares_int += n
-        if self.long:
-            self.num_shares_int -= n
+        if self._short:
+            self._nshares_int += n
+        if self._long:
+            self._nshares_int -= n
 
     def _freeze(self):
-        self.frozen = True
-        self.end_date = self.bt.current_date
-        self.end_event = self.bt.event
-        self.df_cols += ["start_price", "end_date", "end_event"]
+        self._frozen = True
+        self.end_date = self._bt.current_date
+        self.end_event = self._bt.event
 
 
 class Portfolio(MutableSequence):
@@ -290,7 +276,7 @@ class Portfolio(MutableSequence):
         self.bt = bt
 
     @property
-    def value(self) -> float:
+    def total_value(self) -> float:
         """Returns the total value of the portfolio.
         """
         val = 0
@@ -302,7 +288,7 @@ class Portfolio(MutableSequence):
     def df(self) -> pd.DataFrame:
         pos_dict = {}
         for pos in self.positions:
-            for col in pos.df_cols:
+            for col in pos._attr():
                 if col not in pos_dict:
                     pos_dict[col] = []
                 pos_dict[col].append(getattr(pos, col))
@@ -310,20 +296,20 @@ class Portfolio(MutableSequence):
 
     def _get_by_uid(self, uid) -> Position:
         for pos in self.positions:
-            if pos.uid == uid:
+            if pos._uid == uid:
                 return pos
 
     def __repr__(self) -> str:
         return self.positions.__repr__()
 
-    def liquidate(self, num_shares: int = -1, _bt: "Backtester" = None):
+    def liquidate(self, nshares: int = -1, _bt: "Backtester" = None):
         """Liquidate all positions in the current "view" of the portfolio.
         If no view is given using `['some_ticker']`, :meth:`.filter`,
         :meth:`.Portfolio.long` or :meth:`.Portfolio.short`,
         an attempt to liquidate all positions is made.
 
         Args:
-            num_shares:
+            nshares:
                 The number of shares to be liquidated.
                 This should only be used when a ticker is selected using `['some_ticker']`.
 
@@ -334,7 +320,7 @@ class Portfolio(MutableSequence):
 
             Liquidate 10 `MSFT` shares::
 
-                bt.portfolio['MSFT'].liquidate(num_shares=10)
+                bt.portfolio['MSFT'].liquidate(nshares=10)
 
             Liquidate all long positions::
 
@@ -352,13 +338,13 @@ class Portfolio(MutableSequence):
         if bt is None:
             bt = self.bt
             if bt._slippage is not None:
-                self.liquidate(num_shares, bt.lower_bound)
+                self.liquidate(nshares, bt.lower_bound)
         is_long = False
         is_short = False
         for pos in self.positions:
-            if pos.long:
+            if pos._long:
                 is_long = True
-            if pos.short:
+            if pos._short:
                 is_short = True
         if is_long and is_short:
             bt._graceful_stop()
@@ -366,8 +352,8 @@ class Portfolio(MutableSequence):
                 "liquidating a mix of long and short positions is not possible"
             )
         for pos in copy.copy(self.positions):
-            pos = bt.pf._get_by_uid(pos.uid)
-            if num_shares == -1 or num_shares >= pos.num_shares:
+            pos = bt.pf._get_by_uid(pos._uid)
+            if nshares == -1 or nshares >= pos.nshares:
                 bt._available_capital += pos.value
                 if bt._available_capital < 0:
                     bt._graceful_stop()
@@ -379,18 +365,18 @@ class Portfolio(MutableSequence):
                 pos._freeze()
                 bt.trades._add(copy.copy(pos))
 
-                if num_shares != -1:
-                    num_shares -= pos.num_shares
-            elif num_shares > 0 and num_shares < pos.num_shares:
-                bt._available_capital += pos.value_pershare * num_shares
-                pos._remove_shares(num_shares)
+                if nshares != -1:
+                    nshares -= pos.nshares
+            elif nshares > 0 and nshares < pos.nshares:
+                bt._available_capital += pos.value_pershare * nshares
+                pos._remove_shares(nshares)
 
                 hist = copy.copy(pos)
                 hist._freeze()
-                if hist.short:
-                    hist.num_shares_int = (-1) * num_shares
-                if hist.long:
-                    hist.num_shares_int = num_shares
+                if hist._short:
+                    hist._nshares_int = (-1) * nshares
+                if hist._long:
+                    hist._nshares_int = nshares
                 bt.trades._add(hist)
 
                 break
@@ -399,7 +385,7 @@ class Portfolio(MutableSequence):
         self.positions.append(position)
 
     def _remove(self, position):
-        self.positions = [pos for pos in self.positions if pos.uid != position.uid]
+        self.positions = [pos for pos in self.positions if pos._uid != position._uid]
 
     @overload
     def __getitem__(self, index: Union[int, slice]):
@@ -451,7 +437,7 @@ class Portfolio(MutableSequence):
         """
         new_pos = []
         for pos in self.positions:
-            if pos.short:
+            if pos._short:
                 new_pos.append(pos)
         return Portfolio(self.bt, new_pos)
 
@@ -462,7 +448,7 @@ class Portfolio(MutableSequence):
         """
         new_pos = []
         for pos in self.positions:
-            if pos.long:
+            if pos._long:
                 new_pos.append(pos)
         return Portfolio(self.bt, new_pos)
 
@@ -484,8 +470,15 @@ class Portfolio(MutableSequence):
         Args:
             attribute:
                 String name of the attribute to get.
-                Can be any attribute of :clas:`.Position`.
+                Can be any attribute of :class:`.Position`.
         """
+        self.bt._warn.append(
+            f"""
+        .attr will be removed in 0.7
+        you can use b.portfolio.df[{attribute}]
+        instead of b.portfolio.attr('{attribute}')
+        """
+        )
         result = [getattr(pos, attribute) for pos in self.positions]
         if len(result) == 0:
             return None
@@ -626,7 +619,7 @@ class BacktesterBuilder:
         """
         self = copy.deepcopy(self)
         if self.bt._live_plot:
-            self.bt._warn.append(
+            warn(
                 """
                 live plotting and metrics cannot be used together,
                  setting live plotting to false
@@ -646,10 +639,11 @@ class BacktesterBuilder:
 
     def live_plot(
         self,
-        every: int = 10,
+        every: int = None,
         metric: str = "Total Value",
         figsize: Tuple[float, float] = None,
         min_y: int = 0,
+        blocking: bool = False,
     ) -> "BacktesterBuilder":
         """**Optional**, shows the backtest results live using matplotlib.
         Can only be used in notebooks.
@@ -658,26 +652,54 @@ class BacktesterBuilder:
             every:
                 how often metrics should be updated
                 (in events, e.g. 10 = 5 days)
+                the regular default is 10,
+                blocking default is 100
             metric: which metric to plot
             figsize: size of the plot
             min_y:
                 minimum value on the y axis, set to `None`
                 for no lower limit
+            blocking:
+                will disable threading for plots and
+                allow live plotting in terminal,
+                this will slow down the backtester
+                significantly
         """
         self = copy.deepcopy(self)
         if self.bt._live_metrics:
-            self.bt._warn.append(
+            warn(
                 """
                 live metrics and plotting cannot be used together,
                  setting live metrics to false
                 """
             )
             self.bt._live_metrics = False
+        if is_notebook():
+            if every is None:
+                every = 10
+        elif not blocking:
+            warn(
+                """
+                live plots use threading which is not supported
+                with matplotlib outside notebooks. to disable
+                threading for live plots, you can call
+                live_plot with ``blocking = True``.
+
+                live_plot set to false.
+                """
+            )
+            return self
+        elif blocking:
+            self.bt._live_plot_blocking = True
+            if every is None:
+                every = 100
+
         self.bt._live_plot = True
         self.bt._live_plot_every = every
         self.bt._live_plot_metric = metric
         self.bt._live_plot_figsize = figsize
         self.bt._live_plot_min = min_y
+
         return self
 
     def no_live_plot(self) -> "BacktesterBuilder":
@@ -880,6 +902,8 @@ class Backtester:
         self._live_plot_metric = "Total Value"
         self._live_plot_figsize = None
         self._live_plot_min = None
+        self._live_plot_axes = None
+        self._live_plot_blocking = False
         self._live_metrics = False
         self._live_progress = False
 
@@ -909,8 +933,6 @@ class Backtester:
         self._slippage = None
         self._slippage_percent = None
 
-        self._live_plot_axes = None
-
     def _set_self(self, new_self=None):
         if new_self is not None:
             self = new_self
@@ -1098,9 +1120,13 @@ class Backtester:
                 self._live_plot_figsize = (10, 6.5)
 
         if self.add_metric_exists:
-            fig, axes = plt.subplots(2, 1, sharex=True, figsize=self._live_plot_figsize)
+            fig, axes = plt.subplots(
+                2, 1, sharex=True, figsize=self._live_plot_figsize, num=0
+            )
         else:
-            fig, axes = plt.subplots(1, 1, sharex=True, figsize=self._live_plot_figsize)
+            fig, axes = plt.subplots(
+                1, 1, sharex=True, figsize=self._live_plot_figsize, num=0
+            )
             axes = [axes]
 
         if self._live_progress:
@@ -1119,6 +1145,7 @@ class Backtester:
             pass
         if self._live_plot_min is not None:
             axes[0].set_ylim(bottom=self._live_plot_min)
+        plt.tight_layout()
 
         if self.add_metric_exists:
             try:
@@ -1140,6 +1167,8 @@ class Backtester:
 
         plt.draw()
         plt.pause(0.001)
+        if self._live_plot_blocking:
+            plt.clf()  # needed to prevent overlapping tick labels
 
         captions = []
 
@@ -1193,6 +1222,7 @@ class Backtester:
     def _graceful_stop(self):
         if self._last_thread is not None:
             self._last_thread.join()
+            del self._last_thread
         self._plot(self._get_bts(), last=True)
 
     def _order(
@@ -1259,25 +1289,36 @@ class Backtester:
             capital = capital * self._available_capital
         try:
             if shares is None:
-                num_shares, total = self._trade_cost(current_price, capital)
+                fee_dict = self._trade_cost(current_price, capital)
+                nshares, total, fee = (
+                    fee_dict["nshares"],
+                    fee_dict["total"],
+                    fee_dict["fee"],
+                )
             else:
-                num_shares, total = self._trade_cost(
-                    current_price, self._available_capital, num_shares=shares
+                fee_dict = self._trade_cost(
+                    current_price, self._available_capital, nshares=shares
+                )
+                nshares, total, fee = (
+                    fee_dict["nshares"],
+                    fee_dict["total"],
+                    fee_dict["fee"],
                 )
         except Exception as e:
             self._graceful_stop()
             raise e
         if short:
-            num_shares *= -1
-        if num_shares != 0:
+            nshares *= -1
+        if nshares != 0:
             self._available_capital -= total
             pos = Position(
                 self,
                 symbol,
                 self.current_date,
                 self.event,
-                num_shares,
+                nshares,
                 uid,
+                fee,
                 self._slippage_percent,
             )
             self.portfolio._add(pos)
@@ -1469,7 +1510,7 @@ class Backtester:
             new_bt._has_strategies = False
             if self._slippage is not None:
                 self._init_slippage(new_bt)
-                
+
             # this is bad but not bad enough to
             # do anything other than this hotfix
             self._no_iter = True
@@ -1493,17 +1534,22 @@ class Backtester:
     def _plot(self, bts, last=False):
         try:
             if self._live_plot and (self.i % self._live_plot_every == 0 or last):
-                if self._last_thread is None or not self._last_thread.is_alive():
-                    thr = threading.Thread(target=self._show_live_plot, args=(bts,))
-                    thr.start()
-                    self._last_thread = thr
-                if last:
-                    self._last_thread.join()
+                if not self._live_plot_blocking:
+                    if self._last_thread is None or not self._last_thread.is_alive():
+                        thr = threading.Thread(target=self._show_live_plot, args=(bts,))
+                        thr.start()
+                        self._last_thread = thr
+                    if last:
+                        self._last_thread.join()
+                        self._show_live_plot(bts)
+                else:
                     self._show_live_plot(bts)
             if self._live_metrics and (self.i % self._live_metrics_every == 0 or last):
                 self._show_live_metrics(bts)
-            if not (self._live_metrics or self._live_plot) and self._live_progress and (
-                self.i % self._live_progress_every == 0 or last
+            if (
+                not (self._live_metrics or self._live_plot)
+                and self._live_progress
+                and (self.i % self._live_progress_every == 0 or last)
             ):
                 _cls()
                 print(self._show_live_progress())
@@ -1538,6 +1584,8 @@ class Backtester:
             self._show_live_plot(bts, [start, end])
         else:
             self._show_live_plot(bts)
+        if not is_notebook():
+            plt.show()
 
     def run(self):
         """Run the backtesters strategies without using an iterator.
